package Catalyst::Engine::Apache;

use strict;

# 1.27     MP13
# 1.28     MP13
# 1.29     MP13
# 1.2901   MP13
# 1.30     MP13 TBR

# 1.9901   MP19
# 1.9920   MP19
# 1.999020 MP19 RC3
# 1.999021 MP19 RC4

# 1.999022 MP20 RC5
# 1.999023 MP20 RC6

1;

__END__

=head1 NAME

Catalyst::Engine::Apache - Catalyst Apache Engine

=head1 SYNOPSIS

See L<Catalyst>.

=head1 DESCRIPTION

This class will load the correct MP Engine.

=head1 SEE ALSO

L<Catalyst>.

=head1 AUTHOR

Sebastian Riedel, C<sri@cpan.org>
Christian Hansen C<ch@ngmedia.com>

=head1 COPYRIGHT

This program is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
